@@ -218,7 +218,7 @@ orc_code_allocate_codemem (OrcCode *code, int size)
 void
 orc_code_chunk_free (OrcCodeChunk *chunk)
 {
-  if (_orc_compiler_flag_debug) {
+  if (orc_compiler_is_debug ()) {
     /* If debug is turned on, don't free code */
     return;
   }
@@ -242,10 +242,9 @@ orc_code_region_allocate_codemem_dual_map (OrcCodeRegion *region,
   int fd;
   int n;
   char *filename;
-  mode_t mask;
   int exec_prot = PROT_READ | PROT_EXEC;
 
-  if (_orc_compiler_flag_debug)
+  if (orc_compiler_is_debug ())
     exec_prot |= PROT_WRITE;
 
   filename = malloc (strlen ("/orcexec..") +
@@ -255,15 +254,13 @@ orc_code_region_allocate_codemem_dual_map (OrcCodeRegion *region,
     return FALSE;
 
   sprintf(filename, "%s/orcexec.XXXXXX", dir);
-  mask = umask (0066);
   fd = mkstemp (filename);
-  umask (mask);
   if (fd == -1) {
     ORC_WARNING ("failed to create temp file '%s'. err=%i", filename, errno);
     free (filename);
     return FALSE;
   }
-  if (force_unlink || !_orc_compiler_flag_debug) {
+  if (force_unlink || !orc_compiler_is_debug ()) {
     unlink (filename);
   }
 
