@@ -12,7 +12,9 @@
 #include <AvailabilityMacros.h>
 #include <TargetConditionals.h>
 
-#if TARGET_OS_OSX
+/* TARGET_OS_OSX is not defined on older macOS, but is always defined
+   to 0 for Apple mobile targets. */
+#if !defined(TARGET_OS_OSX) || TARGET_OS_OSX
 #include <libkern/OSCacheControl.h>
 #endif
 #endif
@@ -35,6 +37,50 @@
 #include <valgrind/valgrind.h>
 #endif
 
+#if defined(_WIN64)
+#if defined(_M_ARM64)
+#define THUNK_SIZE 20
+typedef struct {
+  orc_uint32 FunctionLength : 18;
+  orc_uint32 Version : 2;
+  orc_uint32 HasExceptionHandler : 1;
+  orc_uint32 PackedEpilog : 1;
+  orc_uint32 EpilogCount : 5;
+  orc_uint32 CodeWords : 5;
+  orc_uint8  UnwindCode[4];
+  orc_uint32 ExceptionHandler;
+} UNWIND_INFO;
+#else // _M_X64
+#define THUNK_SIZE 12
+#define UWOP_SET_FPREG 3
+#define UWOP_PUSH_NONVOL 0
+#define UWOP_REG_RBP 5
+typedef union {
+  struct {
+    orc_uint8 CodeOffset;
+    orc_uint8 UnwindOp : 4;
+    orc_uint8 OpInfo : 4;
+  };
+  orc_uint16 FrameOffset;
+} UNWIND_CODE;
+typedef struct {
+  orc_uint8   Version : 3;
+  orc_uint8   Flags : 5;
+  orc_uint8   SizeOfProlog;
+  orc_uint8   CountOfCodes;
+  orc_uint8   FrameRegister : 4;
+  orc_uint8   FrameOffset : 4;
+  UNWIND_CODE UnwindCode[2];
+  orc_uint32  ExceptionHandler;
+} UNWIND_INFO;
+#endif // defined(_M_ARM64)
+typedef struct {
+  RUNTIME_FUNCTION function_table;
+  UNWIND_INFO unwind_info;
+  orc_uint8 thunk[THUNK_SIZE];
+} OrcUnwindInfo;
+#endif
+
 /**
  * SECTION:orccompiler
  * @title: OrcCompiler
@@ -64,14 +110,30 @@ static int orc_compiler_new_temporary (OrcCompiler *compiler, int size);
 static void orc_compiler_check_sizes (OrcCompiler *compiler);
 
 static char **_orc_compiler_flag_list;
-int _orc_compiler_flag_backup;
-int _orc_compiler_flag_emulate;
-int _orc_compiler_flag_debug;
-int _orc_compiler_flag_randomize;
+static orc_bool _orc_compiler_flag_backup;
+static orc_bool _orc_compiler_flag_emulate;
+static orc_bool _orc_compiler_flag_debug;
+static orc_bool _orc_compiler_flag_randomize;
 
 /* For Windows */
 int _orc_codemem_alignment;
 
+#if defined(_WIN64)
+static DWORD orc_exception_handler(PEXCEPTION_RECORD exceptionRecord,
+                                   PEXCEPTION_REGISTRATION_RECORD _unused,
+                                   PCONTEXT context,
+                                   PEXCEPTION_REGISTRATION_RECORD *_unused2)
+{
+  return ExceptionContinueSearch;
+}
+#endif
+
+orc_bool
+orc_compiler_is_debug ()
+{
+  return _orc_compiler_flag_debug != FALSE;
+}
+
 void
 _orc_compiler_init (void)
 {
@@ -112,16 +174,18 @@ _orc_compiler_init (void)
   _orc_codemem_alignment = 15;
 #endif
 
-#ifdef ORC_WINAPI_ONLY_APP
-  if (!_orc_compiler_flag_backup && !_orc_compiler_flag_emulate) {
-    int can_jit = FALSE;
+  int can_jit = TRUE;
 
+  if (!_orc_compiler_flag_backup && !_orc_compiler_flag_emulate) {
     /* If backup code is not enabled and emulation is not enabled, that means
      * we will do JIT compilation and call orc_code_region_allocate_codemem().
-     * When targeting Windows Store apps, the codeGeneration capability must
+     */
+#if defined(HAVE_CODEMEM_VIRTUALALLOC) && defined(ORC_WINAPI_ONLY_APP)
+    /* When targeting Windows Store apps, the codeGeneration capability must
      * be enabled in the app manifest, or passing PAGE_EXECUTE to
      * VirtualProtectFromApp will return NULL. In this case, we must force
      * backup C code, and if that's not available, we must emulate. */
+    can_jit = FALSE;
     void *mem = VirtualAllocFromApp (NULL, page_size, MEM_COMMIT,
         PAGE_READWRITE);
     if (mem) {
@@ -129,19 +193,35 @@ _orc_compiler_init (void)
       if (VirtualProtectFromApp (mem, page_size, PAGE_EXECUTE, &old_protect) > 0)
         can_jit = TRUE;
       VirtualFree (mem, 0, MEM_RELEASE);
-    }
-
-    if (!can_jit) {
+    } else {
       ORC_WARNING ("Unable to allocate executable pages: using backup code or "
         "emulation: codeGeneration capability isn't set in the app manifest?");
+    }
+#elif defined(HAVE_CODEMEM_MMAP)
+    /* In this case, we need to check that we can mmap pages as executable.
+     * This is not the case under the combination of SELinux and sandboxing
+     * profiles.
+     */
+    can_jit = FALSE;
+    OrcCodeRegion *region = orc_code_region_alloc();
+    if (region) {
+      can_jit = TRUE;
+      // FIXME: the file descriptor should be kept somewhere
+      // Currently the underlying mmap'd pages are leaked
+      free(region);
+    } else {
+      ORC_WARNING ("Unable to allocate executable pages: using backup code or emulation");
+    }
+#endif
+
+    if (!can_jit) {
       _orc_compiler_flag_backup = TRUE;
       _orc_compiler_flag_emulate = TRUE;
     }
   }
-#endif
 }
 
-int
+orc_bool
 orc_compiler_flag_check (const char *flag)
 {
   int i;
@@ -209,54 +289,6 @@ orc_compiler_allocate_register (OrcCompiler *compiler, int data_reg)
   return 0;
 }
 
-/**
- * orc_program_compile:
- * @program: the OrcProgram to compile
- *
- * Compiles an Orc program for the current CPU.  If successful,
- * executable code for the program was generated and can be
- * executed.
- *
- * The return value indicates various levels of success or failure.
- * Success can be determined by checking for a true value of the
- * macro ORC_COMPILE_RESULT_IS_SUCCESSFUL() on the return value.  This
- * indicates that executable code was generated.  If the macro
- * ORC_COMPILE_RESULT_IS_FATAL() on the return value evaluates to
- * true, then there was a syntactical error in the program.  If the
- * result is neither successful nor fatal, the program can still be
- * emulated.
- *
- * Returns: an OrcCompileResult
- */
-OrcCompileResult
-orc_program_compile (OrcProgram *program)
-{
-  return orc_program_compile_for_target (program, orc_target_get_default ());
-}
-
-/**
- * orc_program_compile_for_target:
- * @program: the OrcProgram to compile
- *
- * Compiles an Orc program for the given target, using the
- * default target flags for that target.
- *
- * Returns: an OrcCompileResult
- */
-OrcCompileResult
-orc_program_compile_for_target (OrcProgram *program, OrcTarget *target)
-{
-  unsigned int flags;
-
-  if (target) {
-    flags = target->get_default_flags ();
-  } else {
-    flags = 0;
-  }
-
-  return orc_program_compile_full (program, target, flags);
-}
-
 #if defined(HAVE_CODEMEM_VIRTUALALLOC)
 static orc_bool
 _set_virtual_protect (void * mem, size_t size, int code_protect)
@@ -285,20 +317,9 @@ _set_virtual_protect (void * mem, size_t size, int code_protect)
 }
 #endif
 
-/**
- * orc_program_compile_full:
- * @program: the OrcProgram to compile
- *
- * Compiles an Orc program for the given target, using the
- * given target flags.
- *
- * Returns: an OrcCompileResult
- */
 OrcCompileResult
-orc_program_compile_full (OrcProgram *program, OrcTarget *target,
-    unsigned int flags)
+orc_compiler_compile_program (OrcCompiler *compiler, OrcProgram *program, OrcTarget *target, unsigned int flags)
 {
-  OrcCompiler *compiler;
   int i;
   OrcCompileResult result;
   const char *error_msg;
@@ -321,9 +342,6 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     program->asm_code = NULL;
   }
 
-  compiler = malloc (sizeof(OrcCompiler));
-  memset (compiler, 0, sizeof(OrcCompiler));
-
   if (program->backup_func) {
     program->code_exec = program->backup_func;
   } else {
@@ -335,17 +353,19 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   compiler->target_flags = flags;
 
   {
-    ORC_LOG("variables");
+    ORC_LOG("Program variables");
     for(i=0;i<ORC_N_VARIABLES;i++){
       if (program->vars[i].size > 0) {
-        ORC_LOG("%d: %s size %d type %d alloc %d", i,
+        ORC_LOG("%d: %s size %d type %d alignment %d is_aligned %d alloc %d", i,
             program->vars[i].name,
             program->vars[i].size,
             program->vars[i].vartype,
+            program->vars[i].alignment,
+            program->vars[i].is_aligned,
             program->vars[i].alloc);
       }
     }
-    ORC_LOG("instructions");
+    ORC_LOG("Program instructions");
     for(i=0;i<program->n_insns;i++){
       ORC_LOG("%d: %s %d %d %d %d", i,
           program->insns[i].opcode->name,
@@ -384,23 +404,24 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   orc_compiler_rewrite_vars (compiler);
   if (compiler->error) goto error;
 
-#if 0
   {
-    ORC_ERROR("variables");
+    ORC_LOG("Compiler variables");
     for(i=0;i<ORC_N_VARIABLES;i++){
       if (compiler->vars[i].size > 0) {
-        ORC_ERROR("%d: %s size %d type %d alloc %d [%d,%d]", i,
+        ORC_LOG("%d: %s size %d type %d alignment %d is_aligned %d alloc %d [%d,%d]", i,
             compiler->vars[i].name,
             compiler->vars[i].size,
             compiler->vars[i].vartype,
+            compiler->vars[i].alignment,
+            compiler->vars[i].is_aligned,
             compiler->vars[i].alloc,
             compiler->vars[i].first_use,
             compiler->vars[i].last_use);
       }
     }
-    ORC_ERROR("instructions");
+    ORC_LOG("Compiler instructions");
     for(i=0;i<compiler->n_insns;i++){
-      ORC_ERROR("%d: %s %d %d %d %d", i,
+      ORC_LOG("%d: %s %d %d %d %d", i,
           compiler->insns[i].opcode->name,
           compiler->insns[i].dest_args[0],
           compiler->insns[i].dest_args[1],
@@ -408,7 +429,6 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
           compiler->insns[i].src_args[1]);
     }
   }
-#endif
   program->orccode = orc_code_new ();
 
   program->orccode->is_2d = program->is_2d;
@@ -417,11 +437,11 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   program->orccode->exec = program->code_exec;
 
   program->orccode->n_insns = compiler->n_insns;
-  program->orccode->insns = malloc(sizeof(OrcInstruction) * compiler->n_insns);
+  program->orccode->insns = orc_malloc(sizeof(OrcInstruction) * compiler->n_insns);
   memcpy (program->orccode->insns, compiler->insns,
       sizeof(OrcInstruction) * compiler->n_insns);
 
-  program->orccode->vars = malloc (sizeof(OrcCodeVariable) * ORC_N_COMPILER_VARIABLES);
+  program->orccode->vars = orc_malloc (sizeof(OrcCodeVariable) * ORC_N_COMPILER_VARIABLES);
   memset (program->orccode->vars, 0,
       sizeof(OrcCodeVariable) * ORC_N_COMPILER_VARIABLES);
   for(i=0;i<ORC_N_COMPILER_VARIABLES;i++){
@@ -455,7 +475,7 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   if (compiler->error) goto error;
 
   ORC_INFO("allocating code memory");
-  compiler->code = malloc(65536);
+  compiler->code = orc_malloc(65536);
   compiler->codeptr = compiler->code;
 
   if (compiler->error) goto error;
@@ -467,10 +487,110 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     goto error;
   }
 
-  program->orccode->code_size = compiler->codeptr - compiler->code;
-  orc_code_allocate_codemem (program->orccode, program->orccode->code_size);
+#if defined(_WIN64) && defined(ORC_SUPPORTS_BACKTRACE_FROM_JIT)
+  OrcUnwindInfo table;
+  // The structures must be DWORD aligned in memory.
+  const unsigned char *alignas_offset =
+      (unsigned char *)((DWORD64)(compiler->codeptr + 3) & (~3));
 
-#if defined(__APPLE__) && TARGET_OS_OSX
+  if (compiler->use_frame_pointer) {
+    memset(&table, 0, sizeof(OrcUnwindInfo));
+    const DWORD64 start_of_orcunwindinfo = alignas_offset - compiler->code;
+
+    table.function_table.BeginAddress = 0;
+
+    // The following Arm64 block is based on the Firefox changeset:
+    // https://hg.mozilla.org/mozilla-central/rev/4d932b82695c
+    // which I fixed to work with pure C.
+
+    // The program counter (and the stack pointer, on Arm64) must be modified
+    // for the handler to be recognised as valid.
+#ifdef _M_ARM64
+    table.function_table.UnwindData =
+        start_of_orcunwindinfo + ORC_STRUCT_OFFSET(OrcUnwindInfo, unwind_info);
+
+    // one 32-bit word gives us up to 4 codes
+    table.unwind_info.CodeWords = 1;
+    // alloc_s small stack of size 1*16
+    table.unwind_info.UnwindCode[0] = 0x1;
+    // end
+    table.unwind_info.UnwindCode[1] = 0xe4;
+
+    // xip0/r16 should be safe to clobber: Windows just used it to call the thunk.
+    const orc_uint8 reg = 16;
+
+    const void *handler = (void *)&orc_exception_handler;
+    const uint16_t *addr = (uint16_t *)&handler;
+
+    // Say `handler` is 0x4444333322221111, then:
+    table.thunk[0] = 0xd2800000 | addr[0] << 5 | reg;  // mov  xip0, 1111
+    table.thunk[1] = 0xf2a00000 | addr[1] << 5 | reg;  // movk xip0, 2222 lsl #0x10
+    table.thunk[2] = 0xf2c00000 | addr[2] << 5 | reg;  // movk xip0, 3333 lsl #0x20
+    table.thunk[3] = 0xf2e00000 | addr[3] << 5 | reg;  // movk xip0, 4444 lsl #0x30
+    table.thunk[4] = 0xd61f0000 | reg << 5;                // br xip0
+#else
+    table.function_table.EndAddress = compiler->codeptr - compiler->code;
+#ifdef __MINGW32__
+    // They're the same member, but MSYS2 only defines the former under x86.
+    table.function_table.UnwindData =
+#else
+    table.function_table.UnwindInfoAddress =
+#endif
+        start_of_orcunwindinfo + ORC_STRUCT_OFFSET(OrcUnwindInfo, unwind_info);
+
+    table.unwind_info.Version = 1;
+    // We handle the exception (let the UCRT terminate the app)
+    table.unwind_info.Flags = UNW_FLAG_EHANDLER;
+    table.unwind_info.SizeOfProlog = 8;
+    // See below
+    table.unwind_info.CountOfCodes = 2;
+    // RAX = 50, RBP = 55
+    // https://learn.microsoft.com/en-us/cpp/build/exception-handling-x64?view=msvc-170#operation-info
+    // https://www.felixcloutier.com/x86/mov
+    table.unwind_info.FrameRegister = UWOP_REG_RBP;
+    // No offset from RSP
+    table.unwind_info.FrameOffset = 0;
+    // Bytes 0-4 are the ENDBR64 CET instruction
+    // The docs kind of lie when they say that offsets are not allowed
+    // until SET_FPREG. I guess they apply to OpInfo instead?
+    table.unwind_info.UnwindCode[0].CodeOffset = 5;
+    table.unwind_info.UnwindCode[0].UnwindOp = UWOP_SET_FPREG;
+    table.unwind_info.UnwindCode[0].OpInfo = 0;  // mov rbp, rsp
+    table.unwind_info.UnwindCode[1].CodeOffset = 4;
+    table.unwind_info.UnwindCode[1].UnwindOp = UWOP_PUSH_NONVOL;
+    table.unwind_info.UnwindCode[1].OpInfo = UWOP_REG_RBP; // push rbp
+    // Same as above -- the exception handler is required, and must do
+    // something to actually be dispatched.
+    table.unwind_info.ExceptionHandler =
+        start_of_orcunwindinfo + ORC_STRUCT_OFFSET(OrcUnwindInfo, thunk);
+    // mov %rax, (address of handler as imm64)
+    // jmp %rax
+    static const orc_uint8 thunk[THUNK_SIZE] = {
+      0x48, 0xb8, 0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xe0,
+    };
+    memcpy (&table.thunk, &thunk, THUNK_SIZE);
+    memcpy (&table.thunk[2], &orc_exception_handler, 8);
+#endif
+
+    program->orccode->code_size = (unsigned char *)alignas_offset -
+                                  compiler->code + sizeof(OrcUnwindInfo);
+  } else {
+    program->orccode->code_size = compiler->codeptr - compiler->code;
+  }
+#else
+  program->orccode->code_size = compiler->codeptr - compiler->code;
+#endif
+
+  orc_code_allocate_codemem (program->orccode, program->orccode->code_size);
+  if (program->orccode->chunk == NULL) {
+    program->code_exec = (void *)orc_executor_emulate;
+    program->orccode->exec = (void *)orc_executor_emulate;
+    orc_compiler_error (compiler, "Cannot reserve executable memory, using emulation");
+    compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
+    goto error;
+  }
+
+#if defined(__APPLE__) && (!defined(TARGET_OS_OSX) || TARGET_OS_OSX)
 #if defined(MAC_OS_VERSION_11_0) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_VERSION_11_0
   if (__builtin_available (macOS 11.0, *)) {
     if (pthread_jit_write_protect_supported_np ())
@@ -483,7 +603,36 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   _set_virtual_protect (program->orccode->code, program->orccode->code_size,
       PAGE_READWRITE);
 #endif
-  memcpy (program->orccode->code, compiler->code, program->orccode->code_size);
+#if defined(_WIN64) && defined(ORC_SUPPORTS_BACKTRACE_FROM_JIT)
+  if (compiler->use_frame_pointer) {
+    void *const program_unwind_info =
+        program->orccode->code + (alignas_offset - compiler->code);
+    PRUNTIME_FUNCTION const runtime_function_address =
+          (PRUNTIME_FUNCTION)((orc_uint8*)program_unwind_info +
+                              ORC_STRUCT_OFFSET(OrcUnwindInfo, function_table));
+    const size_t real_code_size = compiler->codeptr - compiler->code;
+    memcpy (program->orccode->code, compiler->code, real_code_size);
+    memcpy (program_unwind_info, &table, sizeof (OrcUnwindInfo));
+    if (RtlAddFunctionTable (runtime_function_address, 1, (DWORD64)program->orccode->code)) {
+      DWORD64 dyn_base = 0;
+      PRUNTIME_FUNCTION const p = RtlLookupFunctionEntry (
+          (DWORD64)program->orccode->code + 20, &dyn_base, NULL);
+      if (p != runtime_function_address) {
+        ORC_ERROR ("Runtime function for program %s %p is bogus "
+                    "(dynbase=%llx, info=%p, expected=%p)",
+                    program->name, program->orccode->code, dyn_base, p,
+                    runtime_function_address);
+      }
+    } else {
+      ORC_WARNING ("Unable to install unwind info for program %s %p",
+                  program->name, program->orccode->code);
+    }
+  } else {
+    memcpy(program->orccode->code, compiler->code, program->orccode->code_size);
+  }
+#else
+  memcpy(program->orccode->code, compiler->code, program->orccode->code_size);
+#endif
 
 #ifdef VALGRIND_DISCARD_TRANSLATIONS
   VALGRIND_DISCARD_TRANSLATIONS (program->orccode->exec,
@@ -494,7 +643,7 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     compiler->target->flush_cache (program->orccode);
   }
 
-#if defined(__APPLE__) && TARGET_OS_OSX
+#if defined(__APPLE__) && (!defined(TARGET_OS_OSX) || TARGET_OS_OSX)
 #if defined(MAC_OS_VERSION_11_0) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_VERSION_11_0
   if (__builtin_available (macOS 11.0, *)) {
     if (pthread_jit_write_protect_supported_np ()) {
@@ -1120,7 +1269,7 @@ orc_compiler_dup_temporary (OrcCompiler *compiler, int var, int j)
 
   compiler->vars[i].vartype = ORC_VAR_TYPE_TEMP;
   compiler->vars[i].size = compiler->vars[var].size;
-  compiler->vars[i].name = malloc (strlen(compiler->vars[var].name) + 10);
+  compiler->vars[i].name = orc_malloc (strlen(compiler->vars[var].name) + 10);
   sprintf(compiler->vars[i].name, "%s.dup%d", compiler->vars[var].name, j);
   compiler->n_dup_vars++;
 
@@ -1134,7 +1283,7 @@ orc_compiler_new_temporary (OrcCompiler *compiler, int size)
 
   compiler->vars[i].vartype = ORC_VAR_TYPE_TEMP;
   compiler->vars[i].size = size;
-  compiler->vars[i].name = malloc (10);
+  compiler->vars[i].name = orc_malloc (10);
   sprintf(compiler->vars[i].name, "tmp%d", i);
   compiler->n_dup_vars++;
 
@@ -1176,7 +1325,7 @@ orc_compiler_append_code (OrcCompiler *p, const char *fmt, ...)
   va_end (varargs);
 
   n = strlen (tmp);
-  p->asm_code = realloc (p->asm_code, p->asm_code_len + n + 1);
+  p->asm_code = orc_realloc (p->asm_code, p->asm_code_len + n + 1);
   memcpy (p->asm_code + p->asm_code_len, tmp, n + 1);
   p->asm_code_len += n;
 }
@@ -1349,12 +1498,20 @@ static void
 orc_compiler_error_valist (OrcCompiler *compiler, const char *fmt,
     va_list args)
 {
-  char *s;
+  char *s = NULL;
 
   if (compiler->error_msg) return;
 
-  s = malloc (ORC_COMPILER_ERROR_BUFFER_SIZE);
-  vsprintf (s, fmt, args);
+#ifdef HAVE_VASPRINTF
+  if (vasprintf (&s, fmt, args) < 0)
+    ORC_ASSERT (0);
+#elif defined(_UCRT)
+  s = orc_malloc (ORC_COMPILER_ERROR_BUFFER_SIZE);
+  vsnprintf_s (s, ORC_COMPILER_ERROR_BUFFER_SIZE, _TRUNCATE, fmt, args);
+#else
+  s = orc_malloc (ORC_COMPILER_ERROR_BUFFER_SIZE);
+  vsnprintf (s, ORC_COMPILER_ERROR_BUFFER_SIZE, fmt, args);
+#endif
   compiler->error_msg = s;
   compiler->error = TRUE;
   compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
