@@ -30,9 +30,21 @@
 #endif
 
 #include <fcntl.h>
+#include <immintrin.h>
 
 #ifdef _MSC_VER
 #  include <intrin.h>
+#elif defined(__GNUC__)
+#if __GNUC__ >= 8 && __GNUC__ < 9
+// https://gcc.gnu.org/bugzilla/show_bug.cgi?id=71659
+#  include <xsaveintrin.h>
+#endif
+#endif
+
+#if !defined(_MSC_VER) || defined(__clang__)
+#define ORC_TARGET_XSAVE __attribute__((target("xsave")))
+#else
+#define ORC_TARGET_XSAVE
 #endif
 
 #include <orc/orcdebug.h>
@@ -300,6 +312,30 @@ orc_x86_cpuid_get_branding_string (void)
   _orc_cpu_name = orc_x86_processor_string;
 }
 
+// Checks if XMM and YMM state are enabled in XCR0.
+// See 14.3 DETECTION OF INTEL® AVX INSTRUCTIONS on the
+// Intel® 64 and IA-32 Architectures Software Developer’s Manual
+#define XSAVE_SUPPORT_XMM 1U << 1
+#define XSAVE_SUPPORT_YMM 1U << 2
+// On a kernel with Gather Data Sampling mitigation, the former will
+// be disabled -- both bits must be enabled, otherwise it'll hit an
+// undefined opcode trap.
+// See https://docs.kernel.org/admin-guide/hw-vuln/gather_data_sampling.html
+#define XSAVE_SUPPORT_AVX (XSAVE_SUPPORT_YMM | XSAVE_SUPPORT_XMM)
+#ifdef ORC_NEEDS_ASM_XSAVE
+static orc_bool check_xcr0_ymm()
+{
+  uint32_t xcr0;
+  __asm__ ("xgetbv" : "=a" (xcr0) : "c" (0) : "%edx" );
+  return ((xcr0 & XSAVE_SUPPORT_AVX) == XSAVE_SUPPORT_AVX);
+}
+#else
+static orc_bool ORC_TARGET_XSAVE check_xcr0_ymm()
+{
+  return (_xgetbv(0) & XSAVE_SUPPORT_AVX) == XSAVE_SUPPORT_AVX;
+}
+#endif
+
 static void
 orc_x86_cpuid_handle_standard_flags (void)
 {
@@ -329,19 +365,30 @@ orc_x86_cpuid_handle_standard_flags (void)
     orc_x86_sse_flags |= ORC_TARGET_SSE_SSE4_2;
   }
 
-  // Linux mitigation for GDS requires checking for XSAVE too.
   // https://bugzilla.mozilla.org/show_bug.cgi?id=1854795
   // https://gitlab.freedesktop.org/gstreamer/orc/-/issues/65
-  const int xsave_enabled = ecx & (1 << 26);
-
-  if (ecx & (1 << 28) && xsave_enabled) {
-    orc_x86_sse_flags |= ORC_TARGET_AVX_AVX;
-  }
+  // check for xsave enabled (bit 26) and xsave available (bit 27)
+  const orc_uint32 xsave_bits = (1U << 26) | (1U << 27);
+  orc_bool osxsave_enabled = (ecx & xsave_bits) == xsave_bits;
+  const orc_bool avx_instructions_supported = (ecx & (1 << 28)) != 0;
 
   get_cpuid (0x00000007, &eax, &ebx, &ecx, &edx);
 
-  if (ebx & (1 << 5) && xsave_enabled) {
-    orc_x86_sse_flags |= ORC_TARGET_AVX_AVX2;
+  const orc_bool avx2_instructions_supported = (ebx & (1 << 5)) != 0;
+
+  // If xgetbv is available, validate XMM and YMM state available
+  if (osxsave_enabled) {
+    osxsave_enabled = check_xcr0_ymm();
+  }
+
+  if (osxsave_enabled) {
+    if (avx_instructions_supported) {
+      orc_x86_sse_flags |= ORC_TARGET_AVX_AVX;
+    }
+
+    if (avx_instructions_supported && avx2_instructions_supported) {
+      orc_x86_sse_flags |= ORC_TARGET_AVX_AVX2;
+    }
   }
 }
 
@@ -585,6 +632,3 @@ orc_mmx_get_cpu_flags(void)
   orc_x86_detect_cpuid ();
   return orc_x86_mmx_flags;
 }
-
-
-
