@@ -106,6 +106,7 @@ orc_target_c_get_asm_preamble (void)
 {
   return "\n"
     "/* begin Orc C target preamble */\n"
+    "#include <math.h>\n"
     "#define ORC_CLAMP(x,a,b) ((x)<(a) ? (a) : ((x)>(b) ? (b) : (x)))\n"
     "#define ORC_ABS(a) ((a)<0 ? -(a) : (a))\n"
     "#define ORC_MIN(a,b) ((a)<(b) ? (a) : (b))\n"
@@ -141,6 +142,8 @@ orc_target_c_get_asm_preamble (void)
     "#define ORC_RESTRICT restrict\n"
     "#elif defined(__GNUC__) && __GNUC__ >= 4\n"
     "#define ORC_RESTRICT __restrict__\n"
+    "#elif defined(_MSC_VER)\n"
+    "#define ORC_RESTRICT __restrict\n"
     "#else\n"
     "#define ORC_RESTRICT\n"
     "#endif\n"
@@ -816,7 +819,7 @@ c_rule_loadpX (OrcCompiler *p, void *user, OrcInstruction *insn)
           dest, insn->src_args[0] - ORC_VAR_P1 + p->program->n_src_vars);
     } else {
       if (size == 8) {
-        ORC_ASM_CODE(p,"    %s = (ex->params[%d] & 0xffffffff) | ((orc_uint64)(ex->params[%d + (ORC_VAR_T1 - ORC_VAR_P1)]) << 32);\n",
+        ORC_ASM_CODE(p,"    %s = (ex->params[%d] & 0xffffffff) | ((orc_uint64)(ex->params[%d + (ORC_N_PARAMS)]) << 32);\n",
             dest, insn->src_args[0], insn->src_args[0]);
       } else {
         ORC_ASM_CODE(p,"    %s = ex->params[%d];\n", dest,
@@ -1221,9 +1224,7 @@ c_rule_div255w (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_int (dest, p, insn, insn->dest_args[0]);
   c_get_name_int (src, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,
-      "    %s = ((orc_uint16)(((orc_uint16)(%s+128)) + (((orc_uint16)(%s+128))>>8)))>>8;\n",
-      dest, src, src);
+  ORC_ASM_CODE(p, "    %s = (((orc_uint16)%s) * 0x8081u) >> 23;\n", dest, src);
 }
 
 static void
@@ -1240,6 +1241,17 @@ c_rule_divluw (OrcCompiler *p, void *user, OrcInstruction *insn)
       dest, src2, src1, src2);
 }
 
+static void
+c_rule_convwf (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src1[40];
+
+  c_get_name_float (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src1, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p,"    %s = (float)%s;\n", dest, src1);
+}
+
 static void
 c_rule_convlf (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -1248,7 +1260,7 @@ c_rule_convlf (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_float (dest, p, insn, insn->dest_args[0]);
   c_get_name_int (src1, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"    %s = %s;\n", dest, src1);
+  ORC_ASM_CODE(p,"    %s = (float)%s;\n", dest, src1);
 }
 
 static void
@@ -1259,7 +1271,7 @@ c_rule_convld (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_float (dest, p, insn, insn->dest_args[0]);
   c_get_name_int (src1, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"    %s = %s;\n", dest, src1);
+  ORC_ASM_CODE(p,"    %s = (double)%s;\n", dest, src1);
 }
 
 static void
@@ -1322,7 +1334,7 @@ c_rule_convdl (OrcCompiler *p, void *user, OrcInstruction *insn)
 
   ORC_ASM_CODE(p, "    {\n");
   ORC_ASM_CODE(p,"       int tmp;\n");
-  ORC_ASM_CODE(p,"       tmp = %s;\n", src);
+  ORC_ASM_CODE(p,"       tmp = (int)%s;\n", src);
   ORC_ASM_CODE(p,"       if (tmp == 0x80000000 && !(%s & ORC_UINT64_C(0x8000000000000000))) tmp = 0x7fffffff;\n", src_i);
   ORC_ASM_CODE(p,"       %s = tmp;\n", dest);
   ORC_ASM_CODE(p, "    }\n");
@@ -1604,5 +1616,6 @@ orc_c_init (void)
   orc_rule_register (rule_set, "mergebw", c_rule_mergebw, NULL);
   orc_rule_register (rule_set, "mergewl", c_rule_mergewl, NULL);
   orc_rule_register (rule_set, "mergelq", c_rule_mergelq, NULL);
+  orc_rule_register (rule_set, "convwf", c_rule_convwf, NULL);
 }
 
