@@ -31,6 +31,7 @@
 
 #include <orc/orcmips.h>
 #include <orc/orcdebug.h>
+#include <orc/orcutils-private.h>
 #include <orc/orcinternal.h>
 #include <stdlib.h>
 #include <string.h>
@@ -100,7 +101,7 @@ orc_compiler_orc_mips_get_default_flags (void)
 {
   unsigned int flags = ORC_TARGET_MIPS_DSP2;
 
-  if (_orc_compiler_flag_debug) {
+  if (orc_compiler_is_debug ()) {
     flags |= ORC_TARGET_MIPS_FRAME_POINTER;
   }
   return flags;
@@ -415,7 +416,7 @@ get_optimised_instruction_order (OrcCompiler *compiler)
   if (compiler->n_insns == 0)
     return NULL;
 
-  instruction_idx = malloc (compiler->n_insns * sizeof(int));
+  instruction_idx = orc_malloc (compiler->n_insns * sizeof(int));
   for (i=0; i<compiler->n_insns; i++)
     instruction_idx[i] = i;
 
