@@ -1513,21 +1513,22 @@ powerpc_rule_convlf (OrcCompiler *p, void *user, OrcInstruction *insn)
 static void
 powerpc_rule_div255w (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
-  int src1 = ORC_SRC_ARG (p, insn, 0);
-  int dest = ORC_DEST_ARG (p, insn, 0);
-  int tmp = orc_compiler_get_temp_reg (p);
-  int tmp2 = orc_compiler_get_temp_reg (p);
-  int tmpc;
+  const int src1 = ORC_SRC_ARG (p, insn, 0);
+  const int dest = ORC_DEST_ARG(p, insn, 0);
+  const int tmp = powerpc_get_constant (p, ORC_CONST_SPLAT_W, 7);
+  const int tmpc = powerpc_get_constant (p, ORC_CONST_SPLAT_W, 0x8081);
 
-  tmpc = powerpc_get_constant (p, ORC_CONST_SPLAT_W, 0x0080);
-  powerpc_emit_VX_2 (p, "vadduhm", 0x10000040, dest, src1, tmpc);
+  {
+    // mulhuw
+    const int perm = powerpc_get_constant_full(p, 0x10110001, 0x14150405,
+        0x18190809, 0x1c1d0c0d);
 
-  ORC_ASM_CODE(p,"  vspltish %s, 8\n", powerpc_get_regname(tmp2));
-  powerpc_emit_VX(p, 0x1000034c, powerpc_regnum(tmp2), 8, 0);
+    powerpc_emit_vmuleuh (p, p->tmpreg, src1, tmpc);
+    powerpc_emit_vmulouh (p, dest, src1, tmpc);
+    powerpc_emit_vperm (p, dest, dest, p->tmpreg, perm);
+  }
 
-  powerpc_emit_VX_2 (p, "vsrh", 0x10000244, tmp, dest, tmp2);
-  powerpc_emit_VX_2 (p, "vadduhm", 0x10000040, dest, dest, tmp);
-  powerpc_emit_VX_2 (p, "vsrh", 0x10000244, dest, dest, tmp2);
+  powerpc_emit_VX_2 (p, "vsrh", 0x10000244, dest, dest, tmp);
 }
 
 static void
