@@ -8,7 +8,7 @@
 #include <string.h>
 
 #include <orc/orc.h>
-#include <orc/orccpuinsn.h>
+#include <orc/orcutils-private.h>
 #include <orc/orcx86.h>
 #include <orc/orcavx.h>
 #include <orc/orcsse.h>
@@ -41,7 +41,7 @@
 // Special instruction that uses 66 but not the escape 0F
 #define ORC_SKIP_ESCAPE (1U << 6)
 
-static const OrcSysOpcode orc_x86_opcodes[] = {
+static const OrcX86Opcode orc_x86_opcodes[] = {
   { "punpcklbw", ORC_X86_INSN_TYPE_MMXM_MMX, 0, ORC_SIMD_PREFIX_MMX, 0x60 },
   { "punpcklwd", ORC_X86_INSN_TYPE_MMXM_MMX, 0, ORC_SIMD_PREFIX_MMX, 0x61 },
   { "punpckldq", ORC_X86_INSN_TYPE_MMXM_MMX, 0, ORC_SIMD_PREFIX_MMX, 0x62 },
@@ -308,7 +308,7 @@ static const OrcSysOpcode orc_x86_opcodes[] = {
 };
 
 static void
-output_opcode (OrcCompiler *p, const OrcSysOpcode *opcode, int size,
+output_opcode (OrcCompiler *p, const OrcX86Opcode *opcode, int size,
     int src, int dest, OrcX86OpcodePrefix reg_type)
 {
   ORC_ASSERT(opcode->code != 0 || opcode->prefix != 0 || opcode->flags != 0);
@@ -565,7 +565,11 @@ orc_x86_insn_output_asm (OrcCompiler *p, OrcX86Insn *xinsn)
       src_op[0] = 0;
       break;
     case ORC_X86_INSN_TYPE_REG_REGM:
-      sprintf(src_op, "%%%s, ", orc_x86_get_regname (operand1));
+      if (xinsn->type == ORC_X86_RM_REG) {
+        sprintf(src_op, "%%%s, ", orc_x86_get_regname_size (operand1, xinsn->size));
+      } else {
+        sprintf(src_op, "%%%s, ", orc_x86_get_regname (operand1));
+      }
       break;
     case ORC_X86_INSN_TYPE_REG8_REGM:
       sprintf(src_op, "%%%s, ", orc_x86_get_regname_8 (operand1));
@@ -714,7 +718,7 @@ orc_x86_insn_output_asm (OrcCompiler *p, OrcX86Insn *xinsn)
     case ORC_X86_INSN_TYPE_REG_REGM:
     case ORC_X86_INSN_TYPE_IMM8_MMX_REG_REV:
       if (xinsn->type == ORC_X86_RM_REG) {
-        sprintf(dst_op, "%%%s", orc_x86_get_regname (xinsn->dest));
+        sprintf(dst_op, "%%%s", orc_x86_get_regname_size (xinsn->dest, xinsn->size));
       } else if (xinsn->type == ORC_X86_RM_MEMOFFSET) {
         sprintf(dst_op, "%d(%%%s)", xinsn->offset,
             orc_x86_get_regname_ptr (p, xinsn->dest));
@@ -1085,7 +1089,7 @@ orc_x86_get_output_insn (OrcCompiler *p)
   OrcX86Insn *xinsn;
   if (p->n_output_insns >= p->n_output_insns_alloc) {
     p->n_output_insns_alloc += 10;
-    p->output_insns = realloc (p->output_insns,
+    p->output_insns = orc_realloc (p->output_insns,
         sizeof(OrcX86Insn) * p->n_output_insns_alloc);
   }
 
@@ -1816,7 +1820,7 @@ void
 orc_x86_emit_cpuinsn_size (OrcCompiler *p, int index, int size, int src, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1830,7 +1834,7 @@ void
 orc_x86_emit_cpuinsn_imm (OrcCompiler *p, int index, int imm, int src, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1846,7 +1850,7 @@ orc_x86_emit_cpuinsn_load_memoffset (OrcCompiler *p, int index, int size,
     int imm, int offset, int src, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1863,7 +1867,7 @@ orc_x86_emit_cpuinsn_store_memoffset (OrcCompiler *p, int index, int size,
     int imm, int offset, int src, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1880,7 +1884,7 @@ orc_x86_emit_cpuinsn_load_memindex (OrcCompiler *p, int index, int size,
     int imm, int offset, int src, int src_index, int shift, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1899,7 +1903,7 @@ orc_x86_emit_cpuinsn_imm_reg (OrcCompiler *p, int index, int size, int imm,
     int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1915,7 +1919,7 @@ orc_x86_emit_cpuinsn_imm_memoffset (OrcCompiler *p, int index, int size,
     int imm, int offset, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1946,7 +1950,7 @@ orc_x86_emit_cpuinsn_reg_memoffset_s (OrcCompiler *p, int index, int size,
     int src, int offset, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1962,7 +1966,7 @@ orc_x86_emit_cpuinsn_memoffset_reg (OrcCompiler *p, int index, int size,
     int offset, int src, int dest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1977,7 +1981,7 @@ void
 orc_x86_emit_cpuinsn_branch (OrcCompiler *p, int index, int label)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -1989,7 +1993,7 @@ void
 orc_x86_emit_cpuinsn_align (OrcCompiler *p, int index, int align_shift)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2000,7 +2004,7 @@ void
 orc_x86_emit_cpuinsn_label (OrcCompiler *p, int index, int label)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2012,7 +2016,7 @@ void
 orc_x86_emit_cpuinsn_none (OrcCompiler *p, int index)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
   int size = 4;
 
   xinsn->opcode_index = index;
@@ -2025,7 +2029,7 @@ orc_x86_emit_cpuinsn_memoffset (OrcCompiler *p, int index, int size,
     int offset, int srcdest)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2041,7 +2045,7 @@ orc_vex_emit_cpuinsn_none (OrcCompiler *p, const int index,
     const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
   int size = 4;
 
   xinsn->opcode_index = index;
@@ -2056,7 +2060,7 @@ orc_vex_emit_cpuinsn_size (OrcCompiler *const p, const int index,
     const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *const opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *const opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2073,7 +2077,7 @@ void
 orc_vex_emit_cpuinsn_imm (OrcCompiler *const p, const int index, const int imm, const int src0, const int src1, const int dest, const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2093,7 +2097,7 @@ orc_vex_emit_cpuinsn_load_memoffset (OrcCompiler *const p, const int index,
     const int src1, const int dest, const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2114,7 +2118,7 @@ orc_vex_emit_cpuinsn_store_memoffset (OrcCompiler *const p, const int index,
     const int dest, const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2132,7 +2136,7 @@ orc_vex_emit_cpuinsn_load_memindex (OrcCompiler *const p, const int index, const
     const int imm, const int offset, const int src, const int src_index, const int shift, int dest, const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
@@ -2153,7 +2157,7 @@ orc_vex_emit_blend_size (OrcCompiler *const p, const int index,
     const int dest, const OrcX86OpcodePrefix prefix)
 {
   OrcX86Insn *xinsn = orc_x86_get_output_insn (p);
-  const OrcSysOpcode *const opcode = orc_x86_opcodes + index;
+  const OrcX86Opcode *const opcode = orc_x86_opcodes + index;
 
   xinsn->opcode_index = index;
   xinsn->opcode = opcode;
